@@ -7,16 +7,16 @@ import sys
 import time
 
 try:
-    raise Exception()
+    from typing import TYPE_CHECKING
 except:
     TYPE_CHECKING = False
 
-PY2 = sys.version_info[0] == 2
-if PY2:
+PY2 = sys.version_info < (3,)
+if not PY2:
+    unicode   = str
+else:
     sys.dont_write_bytecode = True
     unicode = unicode  # noqa: F821  # pylint: disable=undefined-variable,self-assigning-variable
-else:
-    unicode = str
 
 WINDOWS  = (
     [int(x) for x in platform.version().split(".")]
@@ -40,8 +40,8 @@ except:
 class EnvParams(object):
     def __init__(self)  :
         self.t0 = time.time()
-        self.mod = None
-        self.cfg = None
+        self.mod = ""
+        self.cfg = ""
         self.ox = getattr(sys, "oxidized", None)
 
 
