@@ -18,6 +18,7 @@ from .util import (
     IMPLICATIONS,
     META_NOBOTS,
     SQLITE_VER,
+    UNPLICATIONS,
     Pebkac,
     absreal,
     fsenc,
@@ -30,16 +31,6 @@ from .util import (
     unhumanize,
 )
 
-try:
-    raise Exception()
-
-    raise Exception()
-    raise Exception()
-
-    from .util import RootLogger
-except:
-    pass
-
 if TYPE_CHECKING:
     pass
     # Vflags: TypeAlias = dict[str, str | bool | float | list[str]]
@@ -58,18 +49,20 @@ class AXS(object):
         umove   = None,
         udel   = None,
         uget   = None,
+        upget   = None,
     )  :
         self.uread  = set(uread or [])
         self.uwrite  = set(uwrite or [])
         self.umove  = set(umove or [])
         self.udel  = set(udel or [])
         self.uget  = set(uget or [])
+        self.upget  = set(upget or [])
 
     def __repr__(self)  :
         return "AXS({})".format(
             ", ".join(
                 "{}={!r}".format(k, self.__dict__[k])
-                for k in "uread uwrite umove udel uget".split()
+                for k in "uread uwrite umove udel uget upget".split()
             )
         )
 
@@ -293,6 +286,7 @@ class VFS(object):
         self.amove   = {}
         self.adel   = {}
         self.aget   = {}
+        self.apget   = {}
 
         if realpath:
             self.histpath = os.path.join(realpath, ".hist")  # db / thumbcache
@@ -369,7 +363,6 @@ class VFS(object):
 
     def _find(self, vpath )   :
         """return [vfs,remainder]"""
-        vpath = undot(vpath)
         if vpath == "":
             return self, ""
 
@@ -380,13 +373,15 @@ class VFS(object):
             rem = ""
 
         if name in self.nodes:
-            return self.nodes[name]._find(rem)
+            return self.nodes[name]._find(undot(rem))
 
         return self, vpath
 
-    def can_access(self, vpath , uname )      :
-        """can Read,Write,Move,Delete,Get"""
-        vn, _ = self._find(vpath)
+    def can_access(
+        self, vpath , uname 
+    )       :
+        """can Read,Write,Move,Delete,Get,Upget"""
+        vn, _ = self._find(undot(vpath))
         c = vn.axs
         return (
             uname in c.uread or "*" in c.uread,
@@ -394,6 +389,7 @@ class VFS(object):
             uname in c.umove or "*" in c.umove,
             uname in c.udel or "*" in c.udel,
             uname in c.uget or "*" in c.uget,
+            uname in c.upget or "*" in c.upget,
         )
 
     def get(
@@ -405,6 +401,7 @@ class VFS(object):
         will_move  = False,
         will_del  = False,
         will_get  = False,
+        err  = 403,
     )   :
         """returns [vfsnode,fs_remainder] if user has the requested permissions"""
         if ANYWIN:
@@ -414,7 +411,7 @@ class VFS(object):
                     self.log("vfs", "invalid relpath [{}]".format(vpath))
                 raise Pebkac(404)
 
-        vn, rem = self._find(vpath)
+        vn, rem = self._find(undot(vpath))
         c  = vn.axs
 
         for req, d, msg in [
@@ -426,7 +423,7 @@ class VFS(object):
         ]:
             if req and (uname not in d and "*" not in d) and uname != LEELOO_DALLAS:
                 t = "you don't have {}-access for this location"
-                raise Pebkac(403, t.format(msg))
+                raise Pebkac(err, t.format(msg))
 
         return vn, rem
 
@@ -441,11 +438,20 @@ class VFS(object):
 
     def canonical(self, rem , resolve  = True)  :
         """returns the canonical path (fully-resolved absolute fs path)"""
-        rp = self.realpath
+        ap = self.realpath
         if rem:
-            rp += "/" + rem
+            ap += "/" + rem
 
-        return absreal(rp) if resolve else rp
+        return absreal(ap) if resolve else ap
+
+    def dcanonical(self, rem )  :
+        """resolves until the final component (filename)"""
+        ap = self.realpath
+        if rem:
+            ap += "/" + rem
+
+        ad, fn = os.path.split(ap)
+        return os.path.join(absreal(ad), fn)
 
     def ls(
         self,
@@ -562,14 +568,21 @@ class VFS(object):
                 yield x
 
     def zipgen(
-        self, vrem , flt , uname , dots , scandir 
+        self,
+        vrem ,
+        flt ,
+        uname ,
+        dots ,
+        dirs ,
+        scandir ,
+        wrap  = True,
     )     :
 
         # if multiselect: add all items to archive root
         # if single folder: the folder itself is the top-level item
-        folder = "" if flt else (vrem.split("/")[-1] or "top")
+        folder = "" if flt or not wrap else (vrem.split("/")[-1] or "top")
 
-        g = self.walk(folder, vrem, [], uname, [[True]], dots, scandir, False)
+        g = self.walk(folder, vrem, [], uname, [[True, False]], dots, scandir, False)
         for _, _, vpath, apath, files, rd, vd in g:
             if flt:
                 files = [x for x in files if x[0] in flt]
@@ -603,6 +616,21 @@ class VFS(object):
             for f in [{"vp": v, "ap": a, "st": n[1]} for v, a, n in ret]:
                 yield f
 
+            if not dirs:
+                continue
+
+            ts = int(time.time())
+            st = os.stat_result((16877, -1, -1, 1, 1000, 1000, 8, ts, ts, ts))
+            dnames = [n[0] for n in rd]
+            dstats = [n[1] for n in rd]
+            dnames += list(vd.keys())
+            dstats += [st] * len(vd)
+            vpaths = [vpath + "/" + n for n in dnames] if vpath else dnames
+            apaths = [os.path.join(apath, n) for n in dnames]
+            ret2 = list(zip(vpaths, apaths, dstats))
+            for d in [{"vp": v, "ap": a, "st": n} for v, a, n in ret2]:
+                yield d
+
 
 if WINDOWS:
     re_vol = re.compile(r"^([a-zA-Z]:[\\/][^:]*|[^:]*):([^:]*):(.*)$")
@@ -668,7 +696,8 @@ class AuthSrv(object):
 
     def _parse_config_file(
         self,
-        fd ,
+        fp ,
+        cfg_lines ,
         acct  ,
         daxs  ,
         mflags   ,
@@ -678,7 +707,8 @@ class AuthSrv(object):
         vol_src = None
         vol_dst = None
         self.line_ctr = 0
-        for ln in [x.decode("utf-8").strip() for x in fd]:
+        expand_config_file(cfg_lines, fp, "")
+        for ln in cfg_lines:
             self.line_ctr += 1
             if not ln and vol_src is not None:
                 vol_src = None
@@ -707,6 +737,9 @@ class AuthSrv(object):
                 if not vol_dst.startswith("/"):
                     raise Exception('invalid mountpoint "{}"'.format(vol_dst))
 
+                if vol_src.startswith("~"):
+                    vol_src = os.path.expanduser(vol_src)
+
                 # cfg files override arguments and previous files
                 vol_src = absreal(vol_src)
                 vol_dst = vol_dst.strip("/")
@@ -723,12 +756,13 @@ class AuthSrv(object):
                 t = "WARNING (config-file): permission flag 'a' is deprecated; please use 'rw' instead"
                 self.log(t, 1)
 
+            assert vol_dst is not None
             self._read_vol_str(lvl, uname, daxs[vol_dst], mflags[vol_dst])
 
     def _read_vol_str(
         self, lvl , uname , axs , flags  
     )  :
-        if lvl.strip("crwmdg"):
+        if lvl.strip("crwmdgG"):
             raise Exception("invalid volflag: {},{}".format(lvl, uname))
 
         if lvl == "c":
@@ -758,7 +792,9 @@ class AuthSrv(object):
                 ("m", axs.umove),
                 ("d", axs.udel),
                 ("g", axs.uget),
-            ]:
+                ("G", axs.uget),
+                ("G", axs.upget),
+            ]:  # b bb bbb
                 if ch in lvl:
                     al.add(un)
 
@@ -808,7 +844,7 @@ class AuthSrv(object):
 
         if self.args.v:
             # list of src:dst:permset:permset:...
-            # permset is <rwmdg>[,username][,username] or <c>,<flag>[=args]
+            # permset is <rwmdgG>[,username][,username] or <c>,<flag>[=args]
             for v_str in self.args.v:
                 m = re_vol.match(v_str)
                 if not m:
@@ -829,13 +865,16 @@ class AuthSrv(object):
 
         if self.args.c:
             for cfg_fn in self.args.c:
-                with open(cfg_fn, "rb") as f:
-                    try:
-                        self._parse_config_file(f, acct, daxs, mflags, mount)
-                    except:
-                        t = "\n\033[1;31m\nerror in config file {} on line {}:\n\033[0m"
-                        self.log(t.format(cfg_fn, self.line_ctr), 1)
-                        raise
+                lns  = []
+                try:
+                    self._parse_config_file(cfg_fn, lns, acct, daxs, mflags, mount)
+                except:
+                    lns = lns[: self.line_ctr]
+                    slns = ["{:4}: {}".format(n, s) for n, s in enumerate(lns, 1)]
+                    t = "\033[1;31m\nerror @ line {}, included from {}\033[0m"
+                    t = t.format(self.line_ctr, cfg_fn)
+                    self.log("\n{0}\n{1}{0}".format(t, "\n".join(slns)))
+                    raise
 
         # case-insensitive; normalize
         if WINDOWS:
@@ -870,10 +909,11 @@ class AuthSrv(object):
             zv.flags = mflags[dst]
             zv.dbv = None
 
+        assert vfs
         vfs.all_vols = {}
         vfs.get_all_vols(vfs.all_vols)
 
-        for perm in "read write move del get".split():
+        for perm in "read write move del get pget".split():
             axs_key = "u" + perm
             unames = ["*"] + list(acct.keys())
             umap   = {x: [] for x in unames}
@@ -888,7 +928,7 @@ class AuthSrv(object):
         all_users = {}
         missing_users = {}
         for axs in daxs.values():
-            for d in [axs.uread, axs.uwrite, axs.umove, axs.udel, axs.uget]:
+            for d in [axs.uread, axs.uwrite, axs.umove, axs.udel, axs.uget, axs.upget]:
                 for usr in d:
                     all_users[usr] = 1
                     if usr != "*" and usr not in acct:
@@ -1071,7 +1111,12 @@ class AuthSrv(object):
                 if getattr(self.args, k):
                     vol.flags[k] = True
 
-            for ga, vf in [["no_forget", "noforget"], ["magic", "magic"]]:
+            for ga, vf in (
+                ("no_forget", "noforget"),
+                ("no_dupe", "nodupe"),
+                ("magic", "magic"),
+                ("xlink", "xlink"),
+            ):
                 if getattr(self.args, ga):
                     vol.flags[vf] = True
 
@@ -1079,6 +1124,16 @@ class AuthSrv(object):
                 if k1 in vol.flags:
                     vol.flags[k2] = True
 
+            for k1, k2 in UNPLICATIONS:
+                if k1 in vol.flags:
+                    vol.flags[k2] = False
+
+            dbds = "acid|swal|wal|yolo"
+            vol.flags["dbd"] = dbd = vol.flags.get("dbd") or self.args.dbd
+            if dbd not in dbds.split("|"):
+                t = "invalid dbd [{}]; must be one of [{}]"
+                raise Exception(t.format(dbd, dbds))
+
             # default tag cfgs if unset
             if "mte" not in vol.flags:
                 vol.flags["mte"] = self.args.mte
@@ -1175,6 +1230,18 @@ class AuthSrv(object):
                 self.log(t.format(mtp), 1)
                 errors = True
 
+        have_daw = False
+        for vol in vfs.all_vols.values():
+            daw = vol.flags.get("daw") or self.args.daw
+            if daw:
+                vol.flags["daw"] = True
+                have_daw = True
+
+        if have_daw and self.args.no_dav:
+            t = 'volume "/{}" has volflag "daw" (webdav write-access), but --no-dav is set'
+            self.log(t, 1)
+            errors = True
+
         if errors:
             sys.exit(1)
 
@@ -1193,6 +1260,7 @@ class AuthSrv(object):
                 ["  move", "umove"],
                 ["delete", "udel"],
                 ["   get", "uget"],
+                [" upget", "upget"],
             ]:
                 u = list(sorted(getattr(zv.axs, attr)))
                 u = ", ".join("\033[35meverybody\033[0m" if x == "*" else x for x in u)
@@ -1288,10 +1356,11 @@ class AuthSrv(object):
                 raise Exception("volume not found: " + zs)
 
         self.log(str({"users": users, "vols": vols, "flags": flags}))
-        t = "/{}: read({}) write({}) move({}) del({}) get({})"
+        t = "/{}: read({}) write({}) move({}) del({}) get({}) upget({})"
         for k, zv in self.vfs.all_vols.items():
             vc = zv.axs
-            self.log(t.format(k, vc.uread, vc.uwrite, vc.umove, vc.udel, vc.uget))
+            vs = [k, vc.uread, vc.uwrite, vc.umove, vc.udel, vc.uget, vc.upget]
+            self.log(t.format(*vs))
 
         flag_v = "v" in flags
         flag_ln = "ln" in flags
@@ -1316,7 +1385,7 @@ class AuthSrv(object):
                     "",
                     [],
                     u,
-                    [[True]],
+                    [[True, False]],
                     True,
                     not self.args.no_scandir,
                     False,
@@ -1360,3 +1429,33 @@ class AuthSrv(object):
 
         if not flag_r:
             sys.exit(0)
+
+
+def expand_config_file(ret , fp , ipath )  :
+    """expand all % file includes"""
+    fp = absreal(fp)
+    ipath += " -> " + fp
+    ret.append("#\033[36m opening cfg file{}\033[0m".format(ipath))
+    if len(ipath.split(" -> ")) > 64:
+        raise Exception("hit max depth of 64 includes")
+
+    if os.path.isdir(fp):
+        for fn in sorted(os.listdir(fp)):
+            fp2 = os.path.join(fp, fn)
+            if not os.path.isfile(fp2):
+                continue  # dont recurse
+
+            expand_config_file(ret, fp2, ipath)
+        return
+
+    with open(fp, "rb") as f:
+        for ln in [x.decode("utf-8").strip() for x in f]:
+            if ln.startswith("% "):
+                fp2 = ln[1:].strip()
+                fp2 = os.path.join(os.path.dirname(fp), fp2)
+                expand_config_file(ret, fp2, ipath)
+                continue
+
+            ret.append(ln)
+
+    ret.append("#\033[36m closed{}\033[0m".format(ipath))
