@@ -4,14 +4,10 @@ from __future__ import print_function, unicode_literals
 import os
 
 from ..util import SYMTIME, fsdec, fsenc
-from . import path
-
-try:
-    raise Exception()
-except:
-    pass
+from . import path as path
 
 _ = (path,)
+__all__ = ["path"]
 
 # grep -hRiE '(^|[^a-zA-Z_\.-])os\.' . | gsed -r 's/ /\n/g;s/\(/(\n/g' | grep -hRiE '(^|[^a-zA-Z_\.-])os\.' | sort | uniq -c
 # printf 'os\.(%s)' "$(grep ^def bos/__init__.py | gsed -r 's/^def //;s/\(.*//' | tr '\n' '|' | gsed -r 's/.$//')"
@@ -29,15 +25,21 @@ def makedirs(name , mode  = 0o755, exist_ok  = True)  :
     bname = fsenc(name)
     try:
         os.makedirs(bname, mode)
+        return True
     except:
         if not exist_ok or not os.path.isdir(bname):
             raise
+        return False
 
 
 def mkdir(p , mode  = 0o755)  :
     return os.mkdir(fsenc(p), mode)
 
 
+def open(p , *a, **ka)  :
+    return os.open(fsenc(p), *a, **ka)
+
+
 def rename(src , dst )  :
     return os.rename(fsenc(src), fsenc(dst))
 
