@@ -10,13 +10,6 @@ from .__init__ import TYPE_CHECKING
 from .authsrv import AuthSrv
 from .util import HMaccas, Pebkac
 
-try:
-    raise Exception()
-
-    from .util import RootLogger
-except:
-    pass
-
 if TYPE_CHECKING:
     from .httpsrv import HttpSrv
 
@@ -41,12 +34,14 @@ class BrokerCli(object):
     for example resolving httpconn.* in httpcli -- see lines tagged #mypy404
     """
 
+    #log: "RootLogger"
+    #args: argparse.Namespace
+    #asrv: AuthSrv
+    #httpsrv: "HttpSrv"
+    #iphash: HMaccas
+
     def __init__(self)  :
-        self.log  = None
-        self.args  = None
-        self.asrv  = None
-        self.httpsrv  = None
-        self.iphash  = None
+        pass
 
     def ask(self, dest , *args )  :
         return ExceptionalQueue(1)
