@@ -6,14 +6,6 @@ from datetime import datetime
 
 from .bos import bos
 
-try:
-    raise Exception()
-
-    from .util import NamedLogger
-except:
-    pass
-
-
 class StreamArc(object):
     def __init__(
         self,
@@ -25,7 +17,7 @@ class StreamArc(object):
         self.fgen = fgen
 
     def gen(self)    :
-        pass
+        raise Exception("override me")
 
 
 def errdesc(errors  )    :
