@@ -5,6 +5,7 @@ import argparse
 import base64
 import calendar
 import gzip
+import logging
 import os
 import re
 import shlex
@@ -16,15 +17,11 @@ import threading
 import time
 from datetime import datetime, timedelta
 
-try:
-    raise Exception()
+# from inspect import currentframe
+# print(currentframe().f_lineno)
 
-    raise Exception()
-    raise Exception()
-except:
-    pass
 
-from .__init__ import ANYWIN, MACOS, PY2, VT100, WINDOWS, EnvParams, unicode
+from .__init__ import ANYWIN, MACOS, TYPE_CHECKING, VT100, EnvParams, unicode
 from .authsrv import AuthSrv
 from .mtag import HAVE_FFMPEG, HAVE_FFPROBE
 from .tcpsrv import TcpSrv
@@ -32,6 +29,9 @@ from .th_srv import HAVE_PIL, HAVE_VIPS, HAVE_WEBP, ThumbSrv
 from .up2k import Up2k
 from .util import (
     VERSIONS,
+    Daemon,
+    Garda,
+    HLog,
     HMaccas,
     alltrace,
     ansi_re,
@@ -41,6 +41,13 @@ from .util import (
     start_stackmon,
 )
 
+if TYPE_CHECKING:
+    try:
+        from .mdns import MDNS
+        from .ssdp import SSDPd
+    except:
+        pass
+
 
 class SvcHub(object):
     """
@@ -75,8 +82,14 @@ class SvcHub(object):
 
         self.iphash = HMaccas(os.path.join(self.E.cfg, "iphash"), 8)
 
+        # for non-http clients (ftp)
+        self.bans   = {}
+        self.gpwd = Garda(self.args.ban_pw)
+        self.g404 = Garda(self.args.ban_404)
+
         if args.sss or args.s >= 3:
             args.ss = True
+            args.no_dav = True
             args.lo = args.lo or "cpp-%Y-%m%d-%H%M%S.txt.xz"
             args.ls = args.ls or "**,*,ln,p,r"
 
@@ -103,6 +116,11 @@ class SvcHub(object):
         if args.lo:
             self._setup_logfile(printed)
 
+        lg = logging.getLogger()
+        lh = HLog(self.log)
+        lg.handlers = [lh]
+        lg.setLevel(logging.DEBUG)
+
         if args.stackmon:
             start_stackmon(args.stackmon, 0)
 
@@ -137,6 +155,14 @@ class SvcHub(object):
         if args.ls:
             self.asrv.dbg_ls()
 
+        if not ANYWIN:
+            self._setlimits()
+
+        self.log("root", "max clients: {}".format(self.args.nc))
+
+        if not self._process_config():
+            raise Exception("bad config")
+
         self.tcpsrv = TcpSrv(self)
         self.up2k = Up2k(self)
 
@@ -177,10 +203,35 @@ class SvcHub(object):
 
         args.th_poke = min(args.th_poke, args.th_maxage, args.ac_maxage)
 
+        zms = ""
+        if not args.https_only:
+            zms += "d"
+        if not args.http_only:
+            zms += "D"
+
         if args.ftp or args.ftps:
             from .ftpd import Ftpd
 
             self.ftpd = Ftpd(self)
+            zms += "f" if args.ftp else "F"
+
+        if args.smb:
+            # impacket.dcerpc is noisy about listen timeouts
+            sto = socket.getdefaulttimeout()
+            socket.setdefaulttimeout(None)
+
+            from .smbd import SMB
+
+            self.smbd = SMB(self)
+            socket.setdefaulttimeout(sto)
+            self.smbd.start()
+            zms += "s"
+
+        if not args.zms:
+            args.zms = zms
+
+        self.mdns  = None
+        self.ssdp  = None
 
         # decide which worker impl to use
         if self.check_mp_enable():
@@ -222,12 +273,78 @@ class SvcHub(object):
             return
 
         time.sleep(0.1)  # purely cosmetic dw
-        self.log("root", "workers OK\n")
+        if self.tcpsrv.qr:
+            self.log("qr-code", self.tcpsrv.qr)
+        else:
+            self.log("root", "workers OK\n")
+
         self.up2k.init_vols()
 
-        thr = threading.Thread(target=self.sd_notify, name="sd-notify")
-        thr.daemon = True
-        thr.start()
+        Daemon(self.sd_notify, "sd-notify")
+
+    def _process_config(self)  :
+        al = self.args
+
+        al.zm_on = al.zm_on or al.z_on
+        al.zs_on = al.zs_on or al.z_on
+        al.zm_off = al.zm_off or al.z_off
+        al.zs_off = al.zs_off or al.z_off
+        for n in ("zm_on", "zm_off", "zs_on", "zs_off"):
+            vs = getattr(al, n).split(",")
+            vs = [x.strip() for x in vs]
+            vs = [x for x in vs if x]
+            setattr(al, n, vs)
+
+        R = al.rp_loc
+        if "//" in R or ":" in R:
+            t = "found URL in --rp-loc; it should be just the location, for example /foo/bar"
+            raise Exception(t)
+
+        al.R = R = R.strip("/")
+        al.SR = "/" + R if R else ""
+        al.RS = R + "/" if R else ""
+
+        return True
+
+    def _setlimits(self)  :
+        try:
+            import resource
+
+            soft, hard = [
+                x if x > 0 else 1024 * 1024
+                for x in list(resource.getrlimit(resource.RLIMIT_NOFILE))
+            ]
+        except:
+            self.log("root", "failed to read rlimits from os", 6)
+            return
+
+        if not soft or not hard:
+            t = "got bogus rlimits from os ({}, {})"
+            self.log("root", t.format(soft, hard), 6)
+            return
+
+        want = self.args.nc * 4
+        new_soft = min(hard, want)
+        if new_soft < soft:
+            return
+
+        # t = "requesting rlimit_nofile({}), have {}"
+        # self.log("root", t.format(new_soft, soft), 6)
+
+        try:
+            import resource
+
+            resource.setrlimit(resource.RLIMIT_NOFILE, (new_soft, hard))
+            soft = new_soft
+        except:
+            t = "rlimit denied; max open files: {}"
+            self.log("root", t.format(soft), 3)
+            return
+
+        if soft < want:
+            t = "max open files: {} (wanted {} for -nc {})"
+            self.log("root", t.format(soft, want, self.args.nc), 3)
+            self.args.nc = min(self.args.nc, soft // 2)
 
     def _logname(self)  :
         dt = datetime.utcnow()
@@ -252,10 +369,12 @@ class SvcHub(object):
         fn = sel_fn
 
         try:
-            import lzma
-
-            lh = lzma.open(fn, "wt", encoding="utf-8", errors="replace", preset=0)
+            if fn.lower().endswith(".xz"):
+                import lzma
 
+                lh = lzma.open(fn, "wt", encoding="utf-8", errors="replace", preset=0)
+            else:
+                lh = open(fn, "wt", encoding="utf-8", errors="replace")
         except:
             import codecs
 
@@ -269,7 +388,8 @@ class SvcHub(object):
 
         msg = "[+] opened logfile [{}]\n".format(fn)
         printed += msg
-        lh.write("t0: {:.3f}\nargv: {}\n\n{}".format(self.E.t0, " ".join(argv), printed))
+        t = "t0: {:.3f}\nargv: {}\n\n{}"
+        lh.write(t.format(self.E.t0, " ".join(argv), printed))
         self.logf = lh
         self.logf_base_fn = base_fn
         print(msg, end="")
@@ -277,9 +397,25 @@ class SvcHub(object):
     def run(self)  :
         self.tcpsrv.run()
 
-        thr = threading.Thread(target=self.thr_httpsrv_up, name="sig-hsrv-up2")
-        thr.daemon = True
-        thr.start()
+        if getattr(self.args, "zm", False):
+            try:
+                from .mdns import MDNS
+
+                self.mdns = MDNS(self)
+                Daemon(self.mdns.run, "mdns")
+            except:
+                self.log("root", "mdns startup failed;\n" + min_ex(), 3)
+
+        if getattr(self.args, "zs", False):
+            try:
+                from .ssdp import SSDPd
+
+                self.ssdp = SSDPd(self)
+                Daemon(self.ssdp.run, "ssdp")
+            except:
+                self.log("root", "ssdp startup failed;\n" + min_ex(), 3)
+
+        Daemon(self.thr_httpsrv_up, "sig-hsrv-up2")
 
         sigs = [signal.SIGINT, signal.SIGTERM]
         if not ANYWIN:
@@ -294,9 +430,7 @@ class SvcHub(object):
         # never lucky
         if ANYWIN:
             # msys-python probably fine but >msys-python
-            thr = threading.Thread(target=self.stop_thr, name="svchub-sig")
-            thr.daemon = True
-            thr.start()
+            Daemon(self.stop_thr, "svchub-sig")
 
             try:
                 while not self.stop_req:
@@ -316,9 +450,7 @@ class SvcHub(object):
             return "cannot reload; already in progress"
 
         self.reloading = True
-        t = threading.Thread(target=self._reload, name="reloading")
-        t.daemon = True
-        t.start()
+        Daemon(self._reload, "reloading")
         return "reload initiated"
 
     def _reload(self)  :
@@ -341,6 +473,17 @@ class SvcHub(object):
 
         self.shutdown()
 
+    def kill9(self, delay  = 0.0)  :
+        if delay > 0.01:
+            time.sleep(delay)
+            print("component stuck; issuing sigkill")
+            time.sleep(0.1)
+
+        if ANYWIN:
+            os.system("taskkill /f /pid {}".format(os.getpid()))
+        else:
+            os.kill(os.getpid(), signal.SIGKILL)
+
     def signal_handler(self, sig , frame )  :
         if self.stopping:
             if self.nsigs <= 0:
@@ -350,10 +493,7 @@ class SvcHub(object):
                 except:
                     pass
 
-                if ANYWIN:
-                    os.system("taskkill /f /pid {}".format(os.getpid()))
-                else:
-                    os.kill(os.getpid(), signal.SIGKILL)
+                self.kill9()
             else:
                 self.nsigs -= 1
                 return
@@ -380,8 +520,18 @@ class SvcHub(object):
         ret = 1
         try:
             self.pr("OPYTHAT")
-            self.tcpsrv.shutdown()
+            slp = 0.0
+
+            if self.mdns:
+                Daemon(self.mdns.stop)
+                slp = time.time() + 0.5
+
+            if self.ssdp:
+                Daemon(self.ssdp.stop)
+                slp = time.time() + 0.5
+
             self.broker.shutdown()
+            self.tcpsrv.shutdown()
             self.up2k.shutdown()
             if self.thumbsrv:
                 self.thumbsrv.shutdown()
@@ -394,6 +544,14 @@ class SvcHub(object):
                     if n == 3:
                         self.pr("waiting for thumbsrv (10sec)...")
 
+            if hasattr(self, "smbd"):
+                slp = max(slp, time.time() + 0.5)
+                Daemon(self.kill9, a=(1,))
+                Daemon(self.smbd.stop)
+
+            while time.time() < slp:
+                time.sleep(0.1)
+
             self.pr("nailed it", end="")
             ret = self.retcode
         except:
@@ -417,7 +575,7 @@ class SvcHub(object):
 
         with self.log_mutex:
             ts = datetime.utcnow().strftime("%Y-%m%d-%H%M%S.%f")[:-3]
-            self.logf.write("@{} [{}] {}\n".format(ts, src, msg))
+            self.logf.write("@{} [{}\033[0m] {}\n".format(ts, src, msg))
 
             now = time.time()
             if now >= self.next_day:
@@ -480,17 +638,10 @@ class SvcHub(object):
             print(*a, **ka)
 
     def check_mp_support(self)  :
-        vmin = sys.version_info[1]
-        if WINDOWS:
-            msg = "need python 3.3 or newer for multiprocessing;"
-            if PY2 or vmin < 3:
-                return msg
-        elif MACOS:
+        if MACOS:
             return "multiprocessing is wonky on mac osx;"
-        else:
-            msg = "need python 3.3+ for multiprocessing;"
-            if PY2 or vmin < 3:
-                return msg
+        elif sys.version_info < (3, 3):
+            return "need python 3.3 or newer for multiprocessing;"
 
         try:
             x   = mp.Queue(1)
