@@ -107,7 +107,8 @@ var draw_md = (function () {
         map_src = genmap(dom_ref, map_src);
         map_pre = genmap(dom_pre, map_pre);
 
-        clmod(ebi('save'), 'disabled', src == server_md);
+        clmod(ebi('save'), 'disabled',
+            src.replace(/\r/g, "") == server_md.replace(/\r/g, ""));
 
         var t1 = Date.now();
         delay = t1 - t0 > 100 ? 25 : 1;
@@ -230,7 +231,8 @@ redraw = (function () {
 // modification checker
 function Modpoll() {
     var r = {
-        skip_one: true,
+        initial: true,
+        skip_one: false,
         disabled: false
     };
 
@@ -253,7 +255,7 @@ function Modpoll() {
         }
 
         console.log('modpoll...');
-        var url = (document.location + '').split('?')[0] + '?raw&_=' + Date.now();
+        var url = (document.location + '').split('?')[0] + '?_=' + Date.now();
         var xhr = new XHR();
         xhr.open('GET', url, true);
         xhr.responseType = 'text';
@@ -275,8 +277,18 @@ function Modpoll() {
         if (!this.responseText)
             return;
 
-        var server_ref = server_md.replace(/\r/g, '');
-        var server_now = this.responseText.replace(/\r/g, '');
+        var new_md = this.responseText,
+            server_ref = server_md.replace(/\r/g, ''),
+            server_now = new_md.replace(/\r/g, '');
+
+        // firefox bug: sometimes get stale text even if copyparty sent a 200
+        if (r.initial && server_ref != server_now)
+            return modal.confirm('Your browser decided to show an outdated copy of the document!\n\nDo you want to load the latest version from the server instead?', function () {
+                dom_src.value = server_md = new_md;
+                draw_md();
+            }, null);
+
+        r.initial = false;
 
         if (server_ref != server_now) {
             console.log("modpoll diff |" + server_ref.length + "|, |" + server_now.length + "|");
@@ -296,6 +308,7 @@ function Modpoll() {
         console.log('modpoll eq');
     };
 
+    setTimeout(r.periodic, 300);
     if (md_opt.modpoll_freq > 0)
         setInterval(r.periodic, 1000 * md_opt.modpoll_freq);
 
@@ -389,7 +402,7 @@ function save_cb() {
 
 function run_savechk(lastmod, txt, btn, ntry) {
     // download the saved doc from the server and compare
-    var url = (document.location + '').split('?')[0] + '?raw&_=' + Date.now();
+    var url = (document.location + '').split('?')[0] + '?_=' + Date.now();
     var xhr = new XHR();
     xhr.open('GET', url, true);
     xhr.responseType = 'text';
