@@ -1,21 +1,36 @@
 "use strict";
 
-if (!window['console'])
-    window['console'] = {
+if (!window.console || !console.log)
+    window.console = {
         "log": function (msg) { }
     };
 
 
 var wah = '',
+    L, tt, treectl, thegrid, up2k, asmCrypto, hashwasm, vbar, marked,
     CB = '?_=' + Date.now(),
+    R = SR.slice(1),
+    RS = R ? "/" + R : "",
     HALFMAX = 8192 * 8192 * 8192 * 8192,
     HTTPS = (window.location + '').indexOf('https:') === 0,
     TOUCH = 'ontouchstart' in window,
     MOBILE = TOUCH,
     CHROME = !!window.chrome,
+    VCHROME = CHROME ? 1 : 0,
     FIREFOX = ('netscape' in window) && / rv:/.test(navigator.userAgent),
     IPHONE = TOUCH && /iPhone|iPad|iPod/i.test(navigator.userAgent),
-    WINDOWS = navigator.platform ? navigator.platform == 'Win32' : /Windows/.test(navigator.userAgent);
+    LINUX = /Linux/.test(navigator.userAgent),
+    MACOS = /[^a-z]mac ?os/i.test(navigator.userAgent),
+    WINDOWS = /Windows/.test(navigator.userAgent);
+
+if (!window.WebAssembly || !WebAssembly.Memory)
+    window.WebAssembly = false;
+
+if (!window.Notification || !Notification.permission)
+    window.Notification = false;
+
+if (!window.FormData)
+    window.FormData = false;
 
 try {
     CB = '?' + document.currentScript.src.split('?').pop();
@@ -26,8 +41,13 @@ try {
     if (navigator.userAgentData.platform == 'Windows')
         WINDOWS = true;
 
-    if (navigator.userAgentData.brands.some(function (d) { return d.brand == 'Chromium' }))
-        CHROME = true;
+    CHROME = navigator.userAgentData.brands.find(function (d) { return d.brand == 'Chromium' });
+    if (CHROME)
+        VCHROME = CHROME.version;
+    else
+        VCHROME = 0;
+
+    CHROME = !!CHROME;
 }
 catch (ex) { }
 
@@ -38,11 +58,15 @@ var ebi = document.getElementById.bind(document),
     XHR = XMLHttpRequest;
 
 
-function mknod(et, eid) {
+function mknod(et, eid, html) {
     var ret = document.createElement(et);
+
     if (eid)
         ret.id = eid;
 
+    if (html)
+        ret.innerHTML = html;
+
     return ret;
 }
 
@@ -133,7 +157,7 @@ function vis_exh(msg, url, lineNo, columnNo, error) {
     window.onerror = undefined;
     var html = [
         '<h1>you hit a bug!</h1>',
-        '<p style="font-size:1.3em;margin:0">try to <a href="#" onclick="localStorage.clear();location.reload();">reset copyparty settings</a> if you are stuck here, or <a href="#" onclick="ignex();">ignore this</a> / <a href="#" onclick="ignex(true);">ignore all</a></p>',
+        '<p style="font-size:1.3em;margin:0">try to <a href="#" onclick="localStorage.clear();location.reload();">reset copyparty settings</a> if you are stuck here, or <a href="#" onclick="ignex();">ignore this</a> / <a href="#" onclick="ignex(true);">ignore all</a> / <a href="?b=u">basic</a></p>',
         '<p style="color:#fff">please send me a screenshot arigathanks gozaimuch: <a href="<ghi>" target="_blank">github issue</a> or <code>ed#2644</code></p>',
         '<p class="b">' + esc(url + ' @' + lineNo + ':' + columnNo), '<br />' + esc(String(msg)).replace(/\n/g, '<br />') + '</p>',
         '<p><b>UA:</b> ' + esc(navigator.userAgent + '')
@@ -171,8 +195,12 @@ function vis_exh(msg, url, lineNo, columnNo, error) {
         var lsk = Object.keys(ls);
         lsk.sort();
         html.push('<p class="b">');
-        for (var a = 0; a < lsk.length; a++)
+        for (var a = 0; a < lsk.length; a++) {
+            if (ls[lsk[a]].length > 9000)
+                continue;
+
             html.push(' <b>' + esc(lsk[a]) + '</b> <code>' + esc(ls[lsk[a]]) + '</code> ');
+        }
         html.push('</p>');
     }
     catch (e) { }
@@ -229,6 +257,11 @@ function ctrl(e) {
 }
 
 
+function anymod(e, shift_ok) {
+    return e && (e.ctrlKey || e.altKey || e.metaKey || e.isComposing || (!shift_ok && e.shiftKey));
+}
+
+
 function ev(e) {
     e = e || window.event;
     if (!e)
@@ -381,13 +414,14 @@ function clgot(el, cls) {
 
 
 var ANIM = true;
-if (window.matchMedia) {
+try {
     var mq = window.matchMedia('(prefers-reduced-motion: reduce)');
     mq.onchange = function () {
         ANIM = !mq.matches;
     };
     ANIM = !mq.matches;
 }
+catch (ex) { }
 
 
 function yscroll() {
@@ -747,7 +781,7 @@ function lhumantime(v) {
     var t = shumantime(v, 1),
         tp = t.replace(/([a-z])/g, " $1 ").split(/ /g).slice(0, -1);
 
-    if (!window.L || tp.length < 2 || tp[1].indexOf('$') + 1)
+    if (!L || tp.length < 2 || tp[1].indexOf('$') + 1)
         return t;
 
     var ret = '';
@@ -1251,8 +1285,8 @@ var modal = (function () {
         tok, tng, prim, sec, ok_cancel;
 
     r.load = function () {
-        tok = (window.L && L.m_ok) || 'OK';
-        tng = (window.L && L.m_ng) || 'Cancel';
+        tok = (L && L.m_ok) || 'OK';
+        tng = (L && L.m_ng) || 'Cancel';
         prim = '<a href="#" id="modal-ok">' + tok + '</a>';
         sec = '<a href="#" id="modal-ng">' + tng + '</a>';
         ok_cancel = WINDOWS ? prim + sec : sec + prim;
@@ -1606,13 +1640,32 @@ var favico = (function () {
 })();
 
 
+function cprop(name) {
+    return getComputedStyle(document.documentElement).getPropertyValue(name);
+}
+
+
+function bchrome() {
+    console.log(document.documentElement.className);
+    var v, o = QS('meta[name=theme-color]');
+    if (!o)
+        return;
+
+    try {
+        v = cprop('--bg-u3');
+    }
+    catch (ex) { }
+    o.setAttribute('content', v ? v : document.documentElement.className.indexOf('y') + 1 ? '#eee' : '#333');
+}
+bchrome();
+
 var cf_cha_t = 0;
 function xhrchk(xhr, prefix, e404, lvl, tag) {
     if (xhr.status < 400 && xhr.status >= 200)
         return true;
 
     if (xhr.status == 403)
-        return toast.err(0, prefix + (window.L && L.xhr403 || "403: access denied\n\ntry pressing F5, maybe you got logged out"), tag);
+        return toast.err(0, prefix + (L && L.xhr403 || "403: access denied\n\ntry pressing F5, maybe you got logged out"), tag);
 
     if (xhr.status == 404)
         return toast.err(0, prefix + e404, tag);
@@ -1631,7 +1684,7 @@ function xhrchk(xhr, prefix, e404, lvl, tag) {
 
         qsr('#cf_frame');
         var fr = mknod('iframe', 'cf_frame');
-        fr.src = '/?cf_challenge';
+        fr.src = SR + '/?cf_challenge';
         document.body.appendChild(fr);
     }
 
